"use strict";

exports.__esModule = true;
exports["default"] = createTransitionHookMiddleware;

var _isPromise = _interopRequireDefault(require("is-promise"));

var _warning = _interopRequireDefault(require("warning"));

var _ActionTypes = _interopRequireDefault(require("./ActionTypes"));

var _Actions = _interopRequireDefault(require("./Actions"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function runHook(hook, location, callback) {
  var result;

  try {
    result = hook(location);
  } catch (e) {
    process.env.NODE_ENV !== "production" ? (0, _warning["default"])(false, 'Ignoring transition hook `%s` that failed with `%s`.', hook.name, e) : void 0;
    result = null;
  }

  if (!(0, _isPromise["default"])(result)) {
    return callback(result);
  }

  result["catch"](function (e) {
    process.env.NODE_ENV !== "production" ? (0, _warning["default"])(false, 'Ignoring transition hook `%s` that failed with `%s`.', hook.name, e) : void 0;
    return null;
  }).then(callback);
  return undefined;
}

function runHooks(hooks, location, callback) {
  if (!hooks.length) {
    return callback(true);
  }

  return runHook(hooks[0], location, function (result) {
    return result != null ? callback(result) : runHooks(hooks.slice(1), location, callback);
  });
}

function maybeConfirm(result) {
  if (typeof result === 'boolean') {
    return result;
  }

  return window.confirm(result); // eslint-disable-line no-alert
}

function runAllowTransition(hooks, location, callback) {
  return runHooks(hooks, location, function (result) {
    return callback(maybeConfirm(result));
  });
}

function createTransitionHookMiddleware(_ref) {
  var _ref$useBeforeUnload = _ref.useBeforeUnload,
      useBeforeUnload = _ref$useBeforeUnload === void 0 ? false : _ref$useBeforeUnload;
  var nextStep = null;
  var hooks = [];
  var onBeforeUnload = useBeforeUnload ?
  /* istanbul ignore next: not testable with Karma */
  function (event) {
    var syncResult = runHooks(hooks, null, function (result) {
      return result;
    });

    if (syncResult === true || syncResult === undefined) {
      // An asynchronous transition hook usually means there will be a
      //  custom confirm dialog. However, we'll already be showing the
      //  before unload dialog, and there's no way to prevent the custom
      //  dialog from showing. In such cases, the application code will
      //  need to explicitly handle the null location anyway, so don't
      //  potentially show two confirmation dialogs.
      return undefined;
    }

    var resultSafe = syncResult || '';
    event.returnValue = resultSafe; // eslint-disable-line no-param-reassign

    return resultSafe;
  } : null;

  function addHook(hook) {
    // Add the beforeunload event listener only as needed, as its presence
    //  prevents the page from being added to the page navigation cache.
    if (hooks.length === 0 && onBeforeUnload) {
      window.addEventListener('beforeunload', onBeforeUnload);
    }

    hooks.push(hook);
    return function () {
      hooks = hooks.filter(function (item) {
        return item !== hook;
      });

      if (hooks.length === 0 && onBeforeUnload) {
        window.removeEventListener('beforeunload', onBeforeUnload);
      }
    };
  }

  function transitionHookMiddleware(_ref2) {
    var dispatch = _ref2.dispatch;
    return function (next) {
      return function (action) {
        var type = action.type,
            payload = action.payload;

        if (nextStep && type === _ActionTypes["default"].UPDATE_LOCATION) {
          var step = nextStep;
          nextStep = null;
          return step(next, action);
        }

        switch (type) {
          case _ActionTypes["default"].TRANSITION:
            return runAllowTransition(hooks, payload, function (allowTransition) {
              if (!allowTransition) {
                return null;
              } // Skip the repeated transition hook check on UPDATE_LOCATION.


              nextStep = function nextStep(nextNext, nextAction) {
                return nextNext(nextAction);
              };

              return next(action);
            });

          case _ActionTypes["default"].UPDATE_LOCATION:
            {
              // No transition hooks to run.
              if (!hooks.length) {
                return next(action);
              } // This is the initial load. It doesn't make sense to block this
              // transition.


              if (payload.delta === 0) {
                return next(action);
              } // Without delta, we can't restore the location.


              if (payload.delta == null) {
                return runAllowTransition(hooks, payload, function (allowTransition) {
                  return allowTransition ? next(action) : null;
                });
              }

              var finishRunAllowTransition = function finishRunAllowTransition(result) {
                if (!maybeConfirm(result)) {
                  return null;
                } // Release the original UPDATE_LOCATION when the un-rewind
                // happens. We need to do so here to maintain the invariant that
                // the store location only updates after the window location.


                nextStep = function nextStep() {
                  return next(action);
                };

                dispatch(_Actions["default"].go(payload.delta));
                return undefined;
              };

              var sync = true;
              var rewindDone = false;
              var syncResult = runHooks(hooks, payload, function (result) {
                if (sync) {
                  return result;
                }

                if (!rewindDone) {
                  // The rewind hasn't finished yet. Replace the next step hook so
                  // we finish running when that happens.
                  nextStep = function nextStep() {
                    return finishRunAllowTransition(result);
                  };

                  return undefined;
                }

                return finishRunAllowTransition(result);
              });
              sync = false;

              switch (syncResult) {
                case true:
                  // The transition was synchronously allowed, so skip the rewind.
                  return next(action);

                case false:
                  // We're done as soon as the rewind finishes.
                  nextStep = function nextStep() {};

                  break;

                case undefined:
                  // Let the callback from runHooks take care of things.
                  nextStep = function nextStep() {
                    rewindDone = true;
                  };

                  break;

                default:
                  // Show the confirm dialog after the rewind.
                  nextStep = function nextStep() {
                    return finishRunAllowTransition(syncResult);
                  };

              }

              dispatch(_Actions["default"].go(-payload.delta));
              return undefined;
            }

          case _ActionTypes["default"].DISPOSE:
            if (hooks.length > 0 && onBeforeUnload) {
              window.removeEventListener('beforeunload', onBeforeUnload);
            }

            return next(action);

          default:
            return next(action);
        }
      };
    };
  }

  transitionHookMiddleware.addHook = addHook;
  return transitionHookMiddleware;
}

module.exports = exports.default;